@@ -136,7 +136,9 @@ g_registered_type_info_get_g_type (GIRegisteredTypeInfo *info)
   if (type_init == NULL)
     return G_TYPE_NONE;
   else if (!strcmp (type_init, "intern"))
-    return G_TYPE_OBJECT;
+    /* The special string "intern" is used for some types exposed by libgobject
+       (that therefore should be always available) */
+    return g_type_from_name (g_registered_type_info_get_type_name (info));
 
   get_type_func = NULL;
   if (!g_typelib_symbol (rinfo->typelib,
