@@ -86,7 +86,7 @@ class GDumpParser(object):
             if isinstance(node, ast.Function):
                 self._initparse_function(node)
 
-        if self._namespace.name == 'GObject':
+        if self._namespace.name == 'GObject' or self._namespace.name == 'GLib':
             for node in self._namespace.itervalues():
                 if isinstance(node, ast.Record):
                     self._initparse_gobject_record(node)
@@ -133,14 +133,6 @@ class GDumpParser(object):
         for node in to_remove:
             self._namespace.remove(node)
 
-        if self._namespace.name == 'GLib':
-            variant = self._namespace.get('Variant')
-            assert variant is not None
-            variant.add_gtype('GVariant', 'g_variant_get_gtype')
-            variant.c_symbol_prefix = 'variant'
-            # Work around scanner bug
-            variant.disguised = False
-
     # Helper functions
 
     def _execute_binary_get_tree(self):
@@ -172,23 +164,31 @@ blob containing data gleaned from GObject's primitive introspection."""
                 shutil.rmtree(self._binary.tmpdir)
 
     def _create_gobject(self, node):
-        type_name = 'G' + node.name
-        if type_name == 'GObject':
-            parent_gitype = None
-            symbol = 'intern'
-        elif type_name == 'GInitiallyUnowned':
-            parent_gitype = ast.Type(target_giname='GObject.Object')
+        symbol = 'intern'
+        parent_gitype = None
+        if node.name == 'ParamSpec':
+            type_name = 'GParam'
+            # Some function use g_param_value instead
+            c_symbol_prefix = 'param_spec'
+        elif node.name == 'Object':
+            type_name = 'GObject'
+            c_symbol_prefix = 'object'
+        elif node.name == 'InitiallyUnowned':
+            type_name = 'GInitiallyUnowned'
             symbol = 'g_initially_unowned_get_type'
+            parent_gitype = ast.Type(target_giname='GObject.Object')
+            # Not really used, but otherwise InitallyUnowned
+            # gets the static methods that should be in Object
+            c_symbol_prefix = 'initially_unowned'
         else:
             assert False
+
         gnode = ast.Class(node.name, parent_gitype,
                           gtype_name=type_name,
                           get_type=symbol,
-                          c_symbol_prefix='object',
+                          c_symbol_prefix=c_symbol_prefix,
                           is_abstract=True)
-        if type_name == 'GObject':
-            gnode.fields.extend(node.fields)
-        else:
+        if node.name == 'InitiallyUnowned':
             # http://bugzilla.gnome.org/show_bug.cgi?id=569408
             # GInitiallyUnowned is actually a typedef for GObject, but
             # that's not reflected in the GIR, where it appears as a
@@ -197,6 +197,8 @@ blob containing data gleaned from GObject's primitive introspection."""
             # GInitiallyUnowned so that struct offset computation
             # works correctly.
             gnode.fields = self._namespace.get('Object').fields
+        else:
+            gnode.fields.extend(node.fields)
         self._namespace.append(gnode, replace=True)
 
     # Parser
@@ -205,16 +207,14 @@ blob containing data gleaned from GObject's primitive introspection."""
         symbol = func.symbol
         if symbol.startswith('_'):
             return
-        elif symbol.endswith('_get_type'):
+        elif (symbol.endswith('_get_type') or symbol.endswith('_get_gtype')):
             self._initparse_get_type_function(func)
 
     def _initparse_get_type_function(self, func):
-        if self._namespace.name == 'GLib':
-            # No GObjects in GLib
-            return False
-        if (self._namespace.name == 'GObject' and
-            func.symbol in ('g_object_get_type', 'g_initially_unowned_get_type')):
-            # We handle these internally, see _create_gobject
+        if func.symbol in ('g_object_get_type',
+                           'g_initially_unowned_get_type',
+                           'g_variant_get_gtype'):
+            # We handle these internally, see _initparse_gobject_record
             return True
         if func.parameters:
             return False
@@ -230,9 +230,14 @@ blob containing data gleaned from GObject's primitive introspection."""
         return True
 
     def _initparse_gobject_record(self, record):
-        # Special handling for when we're parsing GObject
-        if record.name in ("Object", 'InitiallyUnowned'):
+        # Special handling for when we're parsing GObject / GLib
+        if record.name in ('Object', 'InitiallyUnowned', 'ParamSpec'):
             self._create_gobject(record)
+        elif record.name == 'Variant':
+            self._boxed_types['GVariant'] = ast.Boxed('Variant',
+                                                      gtype_name='GVariant',
+                                                      get_type='intern',
+                                                      c_symbol_prefix='variant')
         elif record.name == 'InitiallyUnownedClass':
             record.fields = self._namespace.get('ObjectClass').fields
             record.disguised = False
@@ -306,14 +311,17 @@ blob containing data gleaned from GObject's primitive introspection."""
         get_type = xmlnode.attrib['get-type']
         (ns, name) = self._transformer.split_csymbol(get_type)
         assert ns is self._namespace
-        if name == 'get_type':
+        if name in ('get_type', '_get_gtype'):
             message.fatal("""The GObject name %r isn't compatibile
 with the configured identifier prefixes:
   %r
 The class would have no name.  Most likely you want to specify a
 different --identifier-prefix.""" % (xmlnode.attrib['name'], self._namespace.identifier_prefixes))
-        assert name.endswith('_get_type')
-        return (get_type, name[:-len('_get_type')])
+        if name.endswith('_get_type'):
+            type_suffix = '_get_type'
+        else:
+            type_suffix = '_get_gtype'
+        return (get_type, name[:-len(type_suffix)])
 
     def _introspect_object(self, xmlnode):
         type_name = xmlnode.attrib['name']
@@ -419,9 +427,7 @@ different --identifier-prefix.""" % (xmlnode.attrib['name'], self._namespace.ide
             node.properties.append(ast.Property(
                 pspec.attrib['name'],
                 ast.Type.create_from_gtype_name(ctype),
-                readable, writable, construct, construct_only,
-                ctype,
-                ))
+                readable, writable, construct, construct_only))
         node.properties = node.properties
 
     def _introspect_signals(self, node, xmlnode):
@@ -454,14 +460,8 @@ different --identifier-prefix.""" % (xmlnode.attrib['name'], self._namespace.ide
         node.parent_chain = parent_types
 
     def _introspect_fundamental(self, xmlnode):
-        # We only care about types that can be instantiatable, other
-        # fundamental types such as the Clutter.Fixed/CoglFixed registers
-        # are not yet interesting from an introspection perspective and
-        # are ignored
-        if not xmlnode.attrib.get('instantiatable', False):
-            return
-
         type_name = xmlnode.attrib['name']
+
         is_abstract = bool(xmlnode.attrib.get('abstract', False))
         (get_type, c_symbol_prefix) = self._split_type_and_symbol_prefix(xmlnode)
         try:
