@@ -346,6 +346,8 @@ usage is void (*_gtk_reserved1)(void);"""
         if OPT_ARRAY_ZERO_TERMINATED in array_values:
             container_type.zeroterminated = array_values.get(
                 OPT_ARRAY_ZERO_TERMINATED) == '1'
+        else:
+            container_type.zeroterminated = False
         length = array_values.get(OPT_ARRAY_LENGTH)
         if length is not None:
             paramname = self._get_validate_parameter_name(parent, length, node)
@@ -535,12 +537,16 @@ usage is void (*_gtk_reserved1)(void);"""
         self._adjust_container_type(parent, node, options)
 
         if (OPT_ALLOW_NONE in options or
+            node.type.target_giname == 'Gio.AsyncReadyCallback' or
             node.type.target_giname == 'Gio.Cancellable'):
             node.allow_none = True
 
         if tag is not None and tag.comment is not None:
             node.doc = tag.comment
 
+        if OPT_SKIP in options:
+            node.skip = True
+
         if options:
             for attribute in options.getall(OPT_ATTRIBUTE):
                 node.attributes.append(attribute.flat())
@@ -696,9 +702,13 @@ usage is void (*_gtk_reserved1)(void);"""
         if not tag:
             return
         t = tag.options.get(OPT_TYPE)
-        if not t:
-            return
-        field.type = self._transformer.create_type_from_user_string(t.one())
+        if t:
+            field.type = self._transformer.create_type_from_user_string(t.one())
+
+        try:
+            self._adjust_container_type(parent, field, tag.options)
+        except AttributeError:
+            pass
 
     def _apply_annotations_property(self, parent, prop):
         prefix = self._get_annotation_name(parent)
@@ -811,10 +821,8 @@ the ones that failed to resolve removed."""
                     node.parent = parent
                     break
             else:
-                if isinstance(node, ast.Interface) or not node.fundamental:
+                if isinstance(node, ast.Interface):
                     node.parent = ast.Type(target_giname='GObject.Object')
-                else:
-                    node.parent = None
             for prop in node.properties:
                 self._transformer.resolve_type(prop.type)
             for sig in node.signals:
@@ -893,7 +901,9 @@ _split_uscored_by_type(text_buffer_try_new) -> (ast.Class(TextBuffer), 'try_new'
     def _pair_function(self, func):
         """Check to see whether a toplevel function should be a
 method or constructor of some type."""
-        if func.symbol.endswith('_get_type') or func.symbol.startswith('_'):
+        if (func.symbol.endswith('_get_type')
+            or func.symbol.endswith('_get_gtype')
+            or func.symbol.startswith('_')):
             return
         (ns, subsymbol) = self._transformer.split_csymbol(func.symbol)
         assert ns == self._namespace
