@@ -6,6 +6,7 @@
 
 #include <string.h>
 
+static void gi_marshalling_tests_boxed_struct_free (GIMarshallingTestsBoxedStruct *struct_);
 
 /* Booleans */
 
@@ -1098,6 +1099,27 @@ gi_marshalling_tests_utf8_full_inout (gchar **utf8)
 }
 
 
+/**
+ * gi_marshalling_tests_init_function:
+ * @n_args: (inout) (allow-none): number of args
+ * @argv: (inout) (array length=n_args) (allow-none): args
+ *
+ * This is like gtk_init().
+ */
+gboolean
+gi_marshalling_tests_init_function (gint *n_args, char ***argv)
+{
+    if (n_args == NULL)
+	return TRUE;
+
+    if (*n_args == 0)
+	return TRUE;
+    (*n_args)--;
+    g_assert (argv != NULL);
+    (*argv)[*n_args] = NULL;
+    return TRUE;
+}
+
 /**
  * gi_marshalling_tests_array_fixed_int_return:
  * Returns: (array fixed-size=4):
@@ -1210,9 +1232,30 @@ gi_marshalling_tests_array_return (gint *length)
     return ints;
 }
 
+/**
+ * gi_marshalling_tests_array_return_etc:
+ * @first:
+ * @length: (out):
+ * @last:
+ * @sum: (out):
+ * Returns: (array length=length):
+ */
+const gint *
+gi_marshalling_tests_array_return_etc (gint first, gint *length, gint last, gint *sum)
+{
+    static gint ints[] = {-1, 0, 1, 2};
+
+    ints[0] = first;
+    ints[3] = last;
+    *sum = first + last;
+    *length = 4;
+    return ints;
+}
+
 /**
  * gi_marshalling_tests_array_in:
  * @ints: (array length=length):
+ * @length:
  */
 void
 gi_marshalling_tests_array_in (const gint *ints, gint length)
@@ -1224,6 +1267,48 @@ gi_marshalling_tests_array_in (const gint *ints, gint length)
     g_assert(ints[3] == 2);
 }
 
+/**
+ * gi_marshalling_tests_array_in_len_before:
+ * @length:
+ * @ints: (array length=length):
+ */
+void
+gi_marshalling_tests_array_in_len_before (gint length, const gint *ints)
+{
+    gi_marshalling_tests_array_in (ints, length);
+}
+
+/**
+ * gi_marshalling_tests_array_in_len_zero_terminated:
+ * @ints: (array length=length zero-terminated=1):
+ * @length:
+ */
+void
+gi_marshalling_tests_array_in_len_zero_terminated (const gint *ints, gint length)
+{
+    g_assert (length == 4);
+
+    g_assert (ints[0] == -1);
+    g_assert (ints[1] == 0);
+    g_assert (ints[2] == 1);
+    g_assert (ints[3] == 2);
+
+    /* One past the end, null terminator */
+    g_assert (ints[4] == 0);
+}
+
+/**
+ * gi_marshalling_tests_array_string_in:
+ * @strings: (array length=length):
+ */
+void
+gi_marshalling_tests_array_string_in (const gchar **strings, gint length)
+{
+    g_assert(length == 2);
+    g_assert(g_strcmp0(strings[0], "foo") == 0);
+    g_assert(g_strcmp0(strings[1], "bar") == 0);
+}
+
 /**
  * gi_marshalling_tests_array_uint8_in:
  * @chars: (array length=length):
@@ -1238,6 +1323,75 @@ gi_marshalling_tests_array_uint8_in (const guint8 *chars, gint length)
     g_assert(chars[3] == 'd');
 }
 
+/**
+ * gi_marshalling_tests_array_struct_in:
+ * @structs: (array length=length):
+ */
+void
+gi_marshalling_tests_array_struct_in (GIMarshallingTestsBoxedStruct **structs, gint length)
+{
+    g_assert(length == 3);
+    g_assert(structs[0]->long_ == 1);
+    g_assert(structs[1]->long_ == 2);
+    g_assert(structs[2]->long_ == 3);
+}
+
+/**
+ * gi_marshalling_tests_array_struct_take_in:
+ * @structs: (array length=length) (transfer full):
+ */
+void
+gi_marshalling_tests_array_struct_take_in (GIMarshallingTestsBoxedStruct **structs, gint length)
+{
+    gi_marshalling_tests_array_struct_in (structs, length);
+
+    /* only really useful if run in valgrind actually */
+    gi_marshalling_tests_boxed_struct_free (structs[0]);
+    gi_marshalling_tests_boxed_struct_free (structs[1]);
+    gi_marshalling_tests_boxed_struct_free (structs[2]);
+    g_free (structs);
+}
+
+/**
+ * gi_marshalling_tests_array_enum_in:
+ * @_enum: (array length=length) (transfer none):
+ * @length:
+ */
+void
+gi_marshalling_tests_array_enum_in (GIMarshallingTestsEnum *_enum, gint length)
+{
+    g_assert (length == 3);
+    g_assert (_enum[0] == GI_MARSHALLING_TESTS_ENUM_VALUE1);
+    g_assert (_enum[1] == GI_MARSHALLING_TESTS_ENUM_VALUE2);
+    g_assert (_enum[2] == GI_MARSHALLING_TESTS_ENUM_VALUE3);
+}
+
+/**
+ * gi_marshalling_tests_array_in_guint64_len:
+ * @ints: (array length=length) (transfer none):
+ * @length:
+ */
+void
+gi_marshalling_tests_array_in_guint64_len (const gint *ints, guint64 length)
+{
+    g_assert (length == 4);
+
+    gi_marshalling_tests_array_in (ints, length);
+}
+
+/**
+ * gi_marshalling_tests_array_in_guint8_len:
+ * @ints: (array length=length) (transfer none):
+ * @length:
+ */
+void
+gi_marshalling_tests_array_in_guint8_len (const gint *ints, guint8 length)
+{
+    g_assert (length == 4);
+
+    gi_marshalling_tests_array_in (ints, length);
+}
+
 /**
  * gi_marshalling_tests_array_out:
  * @ints: (out) (array length=length) (transfer none):
@@ -1251,6 +1405,26 @@ gi_marshalling_tests_array_out (gint **ints, gint *length)
     *ints = values;
 }
 
+/**
+ * gi_marshalling_tests_array_out_etc:
+ * @first:
+ * @ints: (out) (array length=length) (transfer none):
+ * @length: (out):
+ * @last:
+ * @sum: (out):
+ */
+void
+gi_marshalling_tests_array_out_etc (gint first, gint **ints, gint *length, gint last, gint *sum)
+{
+    static gint values[] = {-1, 0, 1, 2};
+
+    values[0] = first;
+    values[3] = last;
+    *sum = first + last;
+    *length = 4;
+    *ints = values;
+}
+
 /**
  * gi_marshalling_tests_array_inout:
  * @ints: (inout) (array length=length) (transfer none):
@@ -1271,6 +1445,32 @@ gi_marshalling_tests_array_inout (gint **ints, gint *length)
     *ints = values;
 }
 
+/**
+ * gi_marshalling_tests_array_inout_etc:
+ * @first:
+ * @ints: (inout) (array length=length) (transfer none):
+ * @length: (inout):
+ * @last:
+ * @sum: (out):
+ */
+void
+gi_marshalling_tests_array_inout_etc (gint first, gint **ints, gint *length, gint last, gint *sum)
+{
+    static gint values[] = {-2, -1, 0, 1, 2};
+
+    g_assert(*length == 4);
+    g_assert((*ints)[0] == -1);
+    g_assert((*ints)[1] == 0);
+    g_assert((*ints)[2] == 1);
+    g_assert((*ints)[3] == 2);
+
+    values[0] = first;
+    values[4] = last;
+    *sum = first + last;
+    *length = 5;
+    *ints = values;
+}
+
 /**
  * gi_marshalling_tests_array_zero_terminated_return:
  * Returns: (array zero-terminated=1) (transfer none):
@@ -1282,6 +1482,29 @@ gi_marshalling_tests_array_zero_terminated_return (void)
     return values;
 }
 
+/**
+ * gi_marshalling_tests_array_zero_terminated_return_struct:
+ * Returns: (array zero-terminated=1) (transfer full):
+ */
+GIMarshallingTestsBoxedStruct **
+gi_marshalling_tests_array_zero_terminated_return_struct (void)
+{
+    GIMarshallingTestsBoxedStruct **ret = (GIMarshallingTestsBoxedStruct**) g_new (gpointer, 4);
+
+    ret[0] = gi_marshalling_tests_boxed_struct_new ();
+    ret[0]->long_ = 42;
+
+    ret[1] = gi_marshalling_tests_boxed_struct_new ();
+    ret[1]->long_ = 43;
+
+    ret[2] = gi_marshalling_tests_boxed_struct_new ();
+    ret[2]->long_ = 44;
+
+    ret[3] = NULL;
+
+    return ret;
+}
+
 /**
  * gi_marshalling_tests_array_zero_terminated_in:
  * @utf8s: (array zero-terminated=1) (transfer none):
@@ -1580,6 +1803,248 @@ gi_marshalling_tests_garray_utf8_full_inout (GArray **array_)
     *array_ = result;
 }
 
+/**
+ * gi_marshalling_tests_gptrarray_int_none_return:
+ * Returns: (element-type gint) (transfer none):
+ */
+GPtrArray *
+gi_marshalling_tests_gptrarray_int_none_return (void)
+{
+    static GPtrArray *parray = NULL;
+    gint i;
+
+    if (parray == NULL) {
+        parray = g_ptr_array_new ();
+        for (i = 0; i < 4; i++) {
+            g_ptr_array_add (parray, GINT_TO_POINTER(i));
+        }
+    }
+
+    return parray;
+}
+/**
+ * gi_marshalling_tests_gptrarray_utf8_none_return:
+ * Returns: (element-type utf8) (transfer none):
+ */
+GPtrArray *
+gi_marshalling_tests_gptrarray_utf8_none_return (void)
+{
+    static GPtrArray *parray = NULL;
+    static gchar *values[] = {"0", "1", "2"};
+    gint i;
+
+    if (parray == NULL) {
+        parray = g_ptr_array_new ();
+        for (i = 0; i < 3; i++)
+            g_ptr_array_add (parray, (gpointer) values[i]);
+    }
+
+    return parray;
+}
+
+/**
+ * gi_marshalling_tests_gptrarray_utf8_container_return:
+ * Returns: (element-type utf8) (transfer container):
+ */
+GPtrArray *
+gi_marshalling_tests_gptrarray_utf8_container_return (void)
+{
+    GPtrArray *parray = NULL;
+    static gchar *values[] = {"0", "1", "2", NULL};
+    gint i;
+
+    parray = g_ptr_array_new ();
+    for (i = 0; values[i]; i++)
+        g_ptr_array_add (parray, (gpointer)values[i]);
+
+    return parray;
+}
+
+/**
+ * gi_marshalling_tests_gptrarray_utf8_full_return:
+ * Returns: (element-type utf8) (transfer full):
+ */
+GPtrArray *
+gi_marshalling_tests_gptrarray_utf8_full_return (void)
+{
+    GPtrArray *parray = NULL;
+    static gchar *values[] = {"0", "1", "2", NULL};
+    gint i;
+
+    parray = g_ptr_array_new ();
+    for (i = 0; values[i]; i++) {
+        gchar *str = g_strdup (values[i]);
+        g_ptr_array_add (parray, (gpointer)str);
+    }
+
+    return parray;
+}
+
+/**
+ * gi_marshalling_tests_gptrarray_int_none_in:
+ * @parray_: (element-type gint) (transfer none):
+ */
+void
+gi_marshalling_tests_gptrarray_int_none_in (GPtrArray *parray_)
+{
+    g_assert (parray_->len == 4);
+    g_assert (g_ptr_array_index (parray_, 0) == GINT_TO_POINTER(0));
+    g_assert (g_ptr_array_index (parray_, 1) == GINT_TO_POINTER(1));
+    g_assert (g_ptr_array_index (parray_, 2) == GINT_TO_POINTER(2));
+    g_assert (g_ptr_array_index (parray_, 3) == GINT_TO_POINTER(3));
+}
+
+/**
+ * gi_marshalling_tests_gptrarray_utf8_none_in:
+ * @parray_: (element-type utf8) (transfer none):
+ */
+void
+gi_marshalling_tests_gptrarray_utf8_none_in (GPtrArray *parray_)
+{
+    g_assert (parray_->len == 3);
+    g_assert (strcmp (g_ptr_array_index (parray_, 0), "0") == 0);
+    g_assert (strcmp (g_ptr_array_index (parray_, 1), "1") == 0);
+    g_assert (strcmp (g_ptr_array_index (parray_, 2), "2") == 0);
+}
+
+/**
+ * gi_marshalling_tests_gptrarray_utf8_none_out:
+ * @parray_: (out) (element-type utf8) (transfer none):
+ */
+void
+gi_marshalling_tests_gptrarray_utf8_none_out (GPtrArray **parray_)
+{
+    static GPtrArray *internal = NULL;
+    static gchar *values[] = {"0", "1", "2", NULL};
+    gint i;
+
+    if (internal == NULL) {
+        internal = g_ptr_array_new ();
+        for (i = 0; values[i]; i++)
+            g_ptr_array_add (internal, (gpointer)values[i]);
+    }
+
+    *parray_ = internal;
+}
+
+/**
+ * gi_marshalling_tests_gptrarray_utf8_container_out:
+ * @parray_: (out) (element-type utf8) (transfer container):
+ */
+void
+gi_marshalling_tests_gptrarray_utf8_container_out (GPtrArray **parray_)
+{
+    static gchar *values[] = {"0", "1", "2", NULL};
+    gint i;
+
+    *parray_ = NULL;
+
+    *parray_ = g_ptr_array_new ();
+    for (i = 0; values[i]; i++)
+        g_ptr_array_add (*parray_, (gpointer)values[i]);
+}
+
+/**
+ * gi_marshalling_tests_gptrarray_utf8_full_out:
+ * @parray_: (out) (element-type utf8) (transfer full):
+ */
+void
+gi_marshalling_tests_gptrarray_utf8_full_out (GPtrArray **parray_)
+{
+    static gchar *values[] = {"0", "1", "2", NULL};
+    gint i;
+
+    *parray_ = NULL;
+
+    *parray_ = g_ptr_array_new ();
+    for (i = 0; values[i]; i++) {
+        gchar *str = g_strdup (values[i]);
+        g_ptr_array_add (*parray_, (gpointer)str);
+    }
+}
+
+/**
+ * gi_marshalling_tests_gptrarray_utf8_none_inout:
+ * @parray_: (inout) (element-type utf8) (transfer none):
+ */
+void
+gi_marshalling_tests_gptrarray_utf8_none_inout (GPtrArray **parray_)
+{
+    static GPtrArray *internal = NULL;
+    static gchar *values[] = {"-2", "-1", "0", "1", NULL};
+    gint i;
+
+    g_assert ((*parray_)->len == 3);
+    g_assert (strcmp (g_ptr_array_index (*parray_, 0), "0") == 0);
+    g_assert (strcmp (g_ptr_array_index (*parray_, 1), "1") == 0);
+    g_assert (strcmp (g_ptr_array_index (*parray_, 2), "2") == 0);
+
+    if (internal == NULL) {
+        internal = g_ptr_array_new ();
+        for (i = 0; values[i]; i++)
+            g_ptr_array_add (internal, (gpointer) values[i]);
+    }
+
+    *parray_ = internal;
+}
+
+/**
+ * gi_marshalling_tests_gptrarray_utf8_container_inout:
+ * @parray_: (inout) (element-type utf8) (transfer container):
+ */
+void
+gi_marshalling_tests_gptrarray_utf8_container_inout (GPtrArray **parray_)
+{
+    static gchar *val1 = "-2";
+    static gchar *val2 = "-1";
+    static gchar *val3 = "0";
+    static gchar *val4 = "1";
+    GPtrArray *result;
+
+    g_assert ((*parray_)->len == 3);
+    g_assert (strcmp (g_ptr_array_index (*parray_, 0), "0") == 0);
+    g_assert (strcmp (g_ptr_array_index (*parray_, 1), "1") == 0);
+    g_assert (strcmp (g_ptr_array_index (*parray_, 2), "2") == 0);
+
+    result = g_ptr_array_new ();
+    g_ptr_array_add (result, (gpointer) val1);
+    g_ptr_array_add (result, (gpointer) val2);
+    g_ptr_array_add (result, (gpointer) val3);
+    g_ptr_array_add (result, (gpointer) val4);
+
+    *parray_ = result;
+}
+
+/**
+ * gi_marshalling_tests_gptrarray_utf8_full_inout:
+ * @parray_: (inout) (element-type utf8) (transfer full):
+ */
+void
+gi_marshalling_tests_gptrarray_utf8_full_inout (GPtrArray **parray_)
+{
+    static gchar *val1 = "-1";
+    static gchar *val2 = "-2";
+    gchar *val;
+    GPtrArray *result;
+
+    g_assert ((*parray_)->len == 3);
+    g_assert (strcmp (g_ptr_array_index (*parray_, 0), "0") == 0);
+    g_assert (strcmp (g_ptr_array_index (*parray_, 1), "1") == 0);
+    g_assert (strcmp (g_ptr_array_index (*parray_, 2), "2") == 0);
+
+    result = g_ptr_array_new ();
+    val = g_strdup (val2);
+    g_ptr_array_add(result, (gpointer) val);
+    val = g_strdup (val1);
+    g_ptr_array_add(result, (gpointer) val);
+    val = g_strdup ("0");
+    g_ptr_array_add(result, (gpointer) val);
+    val = g_strdup ("1");
+    g_ptr_array_add(result, (gpointer) val);
+
+    *parray_ = result;
+}
+
 /**
  * gi_marshalling_tests_bytearray_full_return:
  * Returns: (transfer full):
@@ -1588,14 +2053,10 @@ GByteArray *
 gi_marshalling_tests_bytearray_full_return (void)
 {
     GByteArray *array = NULL;
-    static gchar *values[] = {"0", "1", "2", "3", NULL};
-    gint i;
+    guint8 data[] = {'\0', '1', '\xFF', '3'};
 
     array = g_byte_array_new ();
-    for (i = 0; values[i]; i++) {
-        gchar *str = g_strdup (values[i]);
-        g_byte_array_append (array, (const guint8*)str, 1);
-    }
+    g_byte_array_append (array, (const guint8*)data, G_N_ELEMENTS(data));
 
     return array;
 
@@ -1608,11 +2069,11 @@ gi_marshalling_tests_bytearray_full_return (void)
 void
 gi_marshalling_tests_bytearray_none_in (GByteArray *array_)
 {
-    g_assert (array_->len == 4);
-    g_assert (g_array_index (array_, char, 0) == '0');
-    g_assert (g_array_index (array_, char, 1) == '1');
-    g_assert (g_array_index (array_, char, 2) == '2');
-    g_assert (g_array_index (array_, char, 3) == '3');
+    g_assert_cmpuint (array_->len, ==, 4);
+    g_assert_cmpuint (g_array_index (array_, unsigned char, 0), ==, 0);
+    g_assert_cmpuint (g_array_index (array_, unsigned char, 1), ==, 49);
+    g_assert_cmpuint (g_array_index (array_, unsigned char, 2), ==, 0xFF);
+    g_assert_cmpuint (g_array_index (array_, unsigned char, 3), ==, 51);
 }
 
 /**
