@@ -353,6 +353,8 @@ void gi_marshalling_tests_no_type_flags_inout (GIMarshallingTestsNoTypeFlags *fl
 
 /* Arrays */
 
+gboolean gi_marshalling_tests_init_function (gint *n_args, char ***argv);
+
 /* Fixed-size */
 const gint *gi_marshalling_tests_array_fixed_int_return (void);
 const gshort *gi_marshalling_tests_array_fixed_short_return (void);
@@ -379,7 +381,6 @@ void gi_marshalling_tests_array_uint8_in (const guint8 *chars, gint length);
 void gi_marshalling_tests_array_struct_in (GIMarshallingTestsBoxedStruct **structs, gint length);
 void gi_marshalling_tests_array_struct_take_in (GIMarshallingTestsBoxedStruct **structs, gint length);
 void gi_marshalling_tests_array_enum_in (GIMarshallingTestsEnum *_enum, gint length);
-void gi_marshalling_tests_array_nested_in (GSList **list, gint length);
 void gi_marshalling_tests_array_in_guint64_len (const gint *ints, guint64 length);
 void gi_marshalling_tests_array_in_guint8_len (const gint *ints, guint8 length);
 
@@ -421,6 +422,23 @@ void gi_marshalling_tests_garray_utf8_none_inout (GArray **array_);
 void gi_marshalling_tests_garray_utf8_container_inout (GArray **array_);
 void gi_marshalling_tests_garray_utf8_full_inout (GArray **array_);
 
+/* GPtrArray */
+GPtrArray *gi_marshalling_tests_gptrarray_int_none_return (void);
+GPtrArray *gi_marshalling_tests_gptrarray_utf8_none_return (void);
+GPtrArray *gi_marshalling_tests_gptrarray_utf8_container_return (void);
+GPtrArray *gi_marshalling_tests_gptrarray_utf8_full_return (void);
+
+void gi_marshalling_tests_gptrarray_int_none_in (GPtrArray *parray_);
+void gi_marshalling_tests_gptrarray_utf8_none_in (GPtrArray *parray_);
+
+void gi_marshalling_tests_gptrarray_utf8_none_out (GPtrArray **parray_);
+void gi_marshalling_tests_gptrarray_utf8_container_out (GPtrArray **parray_);
+void gi_marshalling_tests_gptrarray_utf8_full_out (GPtrArray **parray_);
+
+void gi_marshalling_tests_gptrarray_utf8_none_inout (GPtrArray **parray_);
+void gi_marshalling_tests_gptrarray_utf8_container_inout (GPtrArray **parray_);
+void gi_marshalling_tests_gptrarray_utf8_full_inout (GPtrArray **parray_);
+
 /* GByteArray */
 
 GByteArray *gi_marshalling_tests_bytearray_full_return (void);
