@@ -3,6 +3,12 @@
 
 #include <glib-object.h>
 
+typedef enum /*< flags,prefix=ANN >*/
+{
+  ANN_FLAG_FOO = 1,
+  ANN_FLAG_BAR = 2
+} AnnotationBitfield;
+
 /**
  * AnnotationCallback:
  * @in: (in) (transfer none): array of ints
@@ -121,6 +127,7 @@ void     annotation_object_watch_full   (AnnotationObject *object,
                                          AnnotationForeachFunc func,
                                          gpointer user_data,
                                          GDestroyNotify destroy);
+void     annotation_object_hidden_self  (gpointer object);
 
 void     annotation_init                (int              *argc, 
 					 char           ***argv);
@@ -158,5 +165,8 @@ void    annotation_ptr_array (GPtrArray *array);
 
 GObject  * annotation_test_parsing_bug630862 (void);
 
+void annotation_space_after_comment_bug631690 (void);
+
+
 #endif /* __ANNOTATION_OBJECT_H__ */
 
