@@ -35,7 +35,7 @@ time_t regress_test_timet (time_t in);
 GType regress_test_gtype (GType in);
 
 /* utf8 */
-G_CONST_RETURN char *regress_test_utf8_const_return (void);
+const char *regress_test_utf8_const_return (void);
 char *regress_test_utf8_nonconst_return (void);
 void regress_test_utf8_const_in (const char *in);
 void regress_test_utf8_out (char **out);
@@ -79,7 +79,7 @@ void regress_test_array_int_null_out (int **arr, int *len);
 
 /* interface */
 /* GList */
-G_CONST_RETURN GList *regress_test_glist_nothing_return (void);
+const GList *regress_test_glist_nothing_return (void);
 GList *regress_test_glist_nothing_return2 (void);
 GList *regress_test_glist_container_return (void);
 GList *regress_test_glist_everything_return (void);
@@ -89,7 +89,7 @@ void regress_test_glist_null_in(GSList *in);
 void regress_test_glist_null_out(GSList **out_list);
 
 /* GSList */
-G_CONST_RETURN GSList *regress_test_gslist_nothing_return (void);
+const GSList *regress_test_gslist_nothing_return (void);
 GSList *regress_test_gslist_nothing_return2 (void);
 GSList *regress_test_gslist_container_return (void);
 GSList *regress_test_gslist_everything_return (void);
@@ -99,8 +99,8 @@ void regress_test_gslist_null_in(GSList *in);
 void regress_test_gslist_null_out(GSList **out_list);
 
 /* GHashTable */
-G_CONST_RETURN GHashTable *regress_test_ghash_null_return (void);
-G_CONST_RETURN GHashTable *regress_test_ghash_nothing_return (void);
+const GHashTable *regress_test_ghash_null_return (void);
+const GHashTable *regress_test_ghash_nothing_return (void);
 GHashTable *regress_test_ghash_nothing_return2 (void);
 GHashTable *regress_test_ghash_container_return (void);
 GHashTable *regress_test_ghash_everything_return (void);
@@ -130,6 +130,11 @@ cairo_surface_t *regress_test_cairo_surface_full_return (void);
 void regress_test_cairo_surface_none_in (cairo_surface_t *surface);
 void regress_test_cairo_surface_full_out (cairo_surface_t **surface);
 
+GVariant *regress_test_gvariant_i (void);
+GVariant *regress_test_gvariant_s (void);
+GVariant *regress_test_gvariant_asv (void);
+GVariant *regress_test_gvariant_v (void);
+GVariant *regress_test_gvariant_as (void);
 
 /* enums / flags */
 
@@ -137,7 +142,8 @@ typedef enum
 {
   REGRESS_TEST_VALUE1,
   REGRESS_TEST_VALUE2,
-  REGRESS_TEST_VALUE3 = -1
+  REGRESS_TEST_VALUE3 = -1,
+  REGRESS_TEST_VALUE4 = '0'
 } RegressTestEnum;
 
 typedef enum
@@ -160,6 +166,14 @@ GType regress_test_enum_unsigned_get_type (void) G_GNUC_CONST;
 GType regress_test_flags_get_type (void) G_GNUC_CONST;
 #define REGRESS_TEST_TYPE_FLAGS (regress_test_flags_get_type ())
 
+/* this is not registered with GType */
+typedef enum
+{
+  REGRESS_TEST_EVALUE1,
+  REGRESS_TEST_EVALUE2 = 42,
+  REGRESS_TEST_EVALUE3 = '0'
+} RegressTestEnumNoGEnum;
+
 const gchar * regress_test_enum_param(RegressTestEnum e);
 const gchar * regress_test_unsigned_enum_param(RegressTestEnumUnsigned e);
 
@@ -175,6 +189,8 @@ void regress_global_get_flags_out (RegressTestFlags *v);
 /* structures */
 typedef struct _RegressTestStructA RegressTestStructA;
 typedef struct _RegressTestStructB RegressTestStructB;
+typedef struct _RegressTestStructC RegressTestStructC;
+typedef struct _RegressTestStructD RegressTestStructD;
 
 struct _RegressTestStructA
 {
@@ -203,6 +219,24 @@ struct _RegressTestStructC
   GObject *obj;
 };
 
+/* This one has annotated fields */
+/**
+ * RegressTestStructD:
+ * @array1: (array zero-terminated=1):
+ * @array2: (array zero-terminated=1) (element-type RegressTestObj):
+ * @field: (type RegressTestObj):
+ * @list: (element-type RegressTestObj):
+ * @garray: (element-type RegressTestObj):
+ */
+struct _RegressTestStructD
+{
+  RegressTestStructA **array1;
+  gpointer            *array2;
+  gpointer             field;
+  GList               *list;
+  GPtrArray           *garray;
+};
+
 /* plain-old-data boxed types */
 typedef struct _RegressTestSimpleBoxedA RegressTestSimpleBoxedA;
 typedef struct _RegressTestSimpleBoxedB RegressTestSimpleBoxedB;
@@ -215,7 +249,8 @@ struct _RegressTestSimpleBoxedA
   RegressTestEnum some_enum;
 };
 
-GType                   regress_test_simple_boxed_a_get_type     (void);
+/* Intentionally uses _get_gtype */
+GType                   regress_test_simple_boxed_a_get_gtype     (void);
 RegressTestSimpleBoxedA *regress_test_simple_boxed_a_copy         (RegressTestSimpleBoxedA *a);
 gboolean                regress_test_simple_boxed_a_equals       (RegressTestSimpleBoxedA *a,
                                                           RegressTestSimpleBoxedA *other_a);
@@ -321,6 +356,46 @@ gboolean regress_test_obj_torture_signature_1 (RegressTestObj    *obj,
                                        guint       m,
                                        GError    **error);
 
+gboolean regress_test_obj_skip_return_val (RegressTestObj *obj,
+                                           gint            a,
+                                           gint           *out_b,
+                                           gdouble         c,
+                                           gint           *inout_d,
+                                           gint           *out_sum,
+                                           gint            num1,
+                                           gint            num2,
+                                           GError        **error);
+
+gboolean regress_test_obj_skip_param (RegressTestObj *obj,
+                                      gint            a,
+                                      gint           *out_b,
+                                      gdouble         c,
+                                      gint           *inout_d,
+                                      gint           *out_sum,
+                                      gint            num1,
+                                      gint            num2,
+                                      GError        **error);
+
+gboolean regress_test_obj_skip_out_param (RegressTestObj *obj,
+                                          gint            a,
+                                          gint           *out_b,
+                                          gdouble         c,
+                                          gint           *inout_d,
+                                          gint           *out_sum,
+                                          gint            num1,
+                                          gint            num2,
+                                          GError        **error);
+
+gboolean regress_test_obj_skip_inout_param (RegressTestObj *obj,
+                                            gint            a,
+                                            gint           *out_b,
+                                            gdouble         c,
+                                            gint           *inout_d,
+                                            gint           *out_sum,
+                                            gint            num1,
+                                            gint            num2,
+                                            GError        **error);
+
 /* virtual */
 int        regress_test_obj_do_matrix (RegressTestObj *obj, const char *somestr);
 
@@ -609,6 +684,17 @@ typedef enum {
   REGRESS_TEST_PUBLIC_ENUM_AFTER  = 1 << 2,
 } RegressTestPrivateEnum;
 
+typedef struct {
+  gint just_int;
+  gint array[10];
+} RegressTestStructFixedArray;
+
+void regress_test_struct_fixed_array_frob (RegressTestStructFixedArray *str);
+
 #define REGRESS_UTF8_CONSTANT "const \xe2\x99\xa5 utf8"
 
+#ifdef __GNUC__
+#define REGRESS_TEST_GNUC_EXTENSION_STMT(foo) (G_GNUC_EXTENSION ({ int x[10]; x[5] = 42; x[5]; }))
+#endif
+
 #endif /* __GITESTTYPES_H__ */
