@@ -3,6 +3,14 @@ import os
 import os.path
 import sys
 from StringIO import StringIO
+import __builtin__
+
+path=os.getenv('UNINSTALLED_INTROSPECTION_SRCDIR', None)
+assert path is not None
+sys.path.insert(0, path)
+
+# Not correct, but enough to get the tests going uninstalled
+__builtin__.__dict__['DATADIR'] = path
 
 from giscanner.annotationparser import AnnotationParser
 from giscanner.ast import Include, Namespace
@@ -16,6 +24,8 @@ from giscanner.scannermain import process_packages
 currentdir = os.path.dirname(os.path.abspath(sys.argv[0]))
 current_name = os.path.basename(currentdir)
 path = os.path.abspath(os.path.join(currentdir, '..', ''))
+top_srcdir = os.environ['UNINSTALLED_INTROSPECTION_SRCDIR']
+top_builddir = os.environ['TOP_BUILDDIR']
 
 class Options:
     def __init__(self):
@@ -70,6 +80,7 @@ def check(args):
                                output=output)
     logger.enable_warnings(True)
     transformer = Transformer(namespace)
+    transformer.set_include_paths([os.path.join(top_srcdir, 'gir'), top_builddir])
     transformer.register_include(Include.from_string("GObject-2.0"))
 
     ss = SourceScanner()
@@ -91,7 +102,7 @@ def check(args):
     main = MainTransformer(transformer, blocks)
     main.transform()
 
-    final = IntrospectablePass(transformer)
+    final = IntrospectablePass(transformer, blocks)
     final.validate()
 
     raw = output.getvalue()
@@ -105,8 +116,12 @@ def check(args):
         warnings.remove('')
     if len(expected_warnings) != len(warnings):
         raise SystemExit(
-            "ERROR: expected %d warnings, but got %d: %r\n" % (
-            len(expected_warnings), len(warnings), warnings))
+            "ERROR in %r: expected %d warnings, but got %d:\n"
+            "----\nexpected:\n%s\n----\ngot:\n%s\n----" % (
+            os.path.basename(filename),
+            len(expected_warnings), len(warnings),
+            '\n'.join([w[1] for w in expected_warnings]),
+            '\n'.join([w.split(':', 2)[2][1:] for w in warnings])))
     for warning, (sort_key, expected) in zip(warnings, expected_warnings):
         actual = warning.split(":", 1)[1]
         if _diff(expected, actual, filename):
